"""
A set of functions and demos to make machine learning projects easier to understand through effective visualizations.
"""

import MLVisualizationTools.modelanalytics as Analytics
import MLVisualizationTools.modelinterface as Interfaces
import MLVisualizationTools.colorizers as Colorizers
import MLVisualizationTools.graphinterface as Graphs
import MLVisualizationTools.datainterface as DataInterfaces
import MLVisualizationTools.types as types