class GraphDataTypes:
    Grid = 'Grid'
    Animation = 'Animation'
    ModelPrediction = 'ModelPrediction'
    DataValues = 'DataValues'

class GraphOutputTypes:
    Auto = 'Auto'
    Matplotlib = 'Matplotlib'
    Plotly = 'Plotly'

class ColorizerModes:
    NotColorized = 'NotColorized'
    Simple = 'Simple'
    Binary = 'Binary'